/*
 *  Gtk-Parted - a front-end for libparted
 *  Copyright (C) 2000, 2001  Damien Genet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * GUI functions
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

//#include <ctype.h>
//#include <sys/stat.h>
#include <errno.h>

#include <parted/parted.h>
#include <gtk/gtk.h>

#include "../misc/misc.h"

#include "gped.h"
#include "ctreeParted.h"
#include "popup.h"
#include "fstab.h"


static gpedDeviceInfo*
_gpedDeviceInfo_get_by_disk            (gpedDeviceInfo *dev_list,
                                        PedDisk        *disk)
{
	gpedDeviceInfo	*walk;
	
	g_return_val_if_fail (dev_list, NULL);
	g_return_val_if_fail (disk, NULL);
	
	for (walk = dev_list; walk; walk = walk->next) {
		if (walk->disk==disk)
			break;
	}
	return walk;
}

gpedDeviceInfo*
_gpedDeviceInfo_get_by_dev_name        (gpedDeviceInfo *dev_list,
                                        gchar          *dev_name)
{
	gpedDeviceInfo	*walk;
	
	g_return_val_if_fail (dev_list, NULL);
	g_return_val_if_fail (dev_name, NULL);
	
	for (walk = dev_list; walk; walk = walk->next) {
		if ( !walk->disk )
			continue;
		if ( str_are_equal (walk->disk->dev->path, dev_name) )
			break;
	}
	return walk;
}

static gpedDeviceInfo*
_gpedDeviceInfo_append                 (gpedDeviceInfo *dev_list,
                                        gpedDeviceInfo *devInfo)
{
	g_return_val_if_fail (devInfo, NULL);
	
	devInfo->prev = NULL;

	if (!dev_list) {
		devInfo->next = NULL;
	} else {
		dev_list->prev = devInfo;
		devInfo->next = dev_list;
	}
	
	return devInfo;
}

static gpedDeviceInfo*
_gpedDeviceInfo_new                    (void)
{
	gpedDeviceInfo	*devInfo;

	if ( !(devInfo = g_malloc (sizeof (gpedDeviceInfo))) ) {
		g_printerr ("%s: allocationg gpedDeviceInfo.\n",
		            g_strerror (errno));
		return NULL;
	}

	return devInfo;
}

static gpedDeviceInfo*
_recursive_build_list                  (PedDevice      *dev)
{
	gpedDeviceInfo	*devInfo;
	
	if ( !(dev = ped_device_get_next (dev)) )
		return NULL;
	
	if ( !(devInfo = _gpedDeviceInfo_new ()) )
		return NULL;
	
	devInfo->dev = dev;
	devInfo->disk = ped_disk_open (dev);
	devInfo->modified = FALSE;
	
	return  _gpedDeviceInfo_append (_recursive_build_list (dev),
	                                devInfo);
}

static gpedDeviceInfo*
_build_device_list                     (gint            devices_count,
                                        gchar          *devices[])
{
	gpedDeviceInfo	*dev_list;
	gint		 i;

	ped_device_probe_all ();
	for (i = 0; i < devices_count; i++) {
		printf ("      add device : %s\n", devices[i]);
		ped_device_get (devices[i]); 
	}
	
	dev_list = _recursive_build_list (NULL);
	
	if (!dev_list) {	/* no disk */
		ui_dialog_opt_create_and_run (_("No device found\nMaybe you don't have privilege to acces disk\nTry running Gtk-Parted as root"), DIALOG_CANCEL);
		return NULL;
	}
	
	return dev_list;
}


PedPartition*
gped_get_part                          (gchar          *dev_name,
                                        guint           part_minor,
                                        gpedDeviceInfo *dev_list)
{
	gpedDeviceInfo	*devInfo;
	
	if ( !(devInfo = _gpedDeviceInfo_get_by_dev_name (dev_list, dev_name)) )
		return NULL;
	
	return ped_disk_get_partition (devInfo->disk, part_minor);
}



static PedExceptionOption
_exception_handler                     (PedException   *exception)
{
	gchar		*message;
	dialog_option	 opt = 0;
	dialog_option	 result;

	g_return_val_if_fail (exception, 0);
	
	/*
	 * for the moment this could seems unusfull,
	 *  but dialog_option may change
	 */
	if (exception->options & PED_EXCEPTION_FIX)
		opt += DIALOG_FIX;
	if (exception->options & PED_EXCEPTION_YES)
		opt += DIALOG_YES;
	if (exception->options & PED_EXCEPTION_NO)
		opt += DIALOG_NO; 
	if (exception->options & PED_EXCEPTION_OK)
		opt += DIALOG_OK;
	if (exception->options & PED_EXCEPTION_RETRY)
		opt += DIALOG_RETRY;
	if (exception->options & PED_EXCEPTION_IGNORE)
		opt += DIALOG_IGNORE;
	if (exception->options & PED_EXCEPTION_CANCEL)
		opt += DIALOG_CANCEL;

	message = g_strconcat (ped_exception_get_type_string (exception->type),
	                       "\n", exception->message, NULL);
	gtk_thread_dec_ref_count ();
	result = ui_dialog_opt_create_and_run (message, opt);
	gtk_thread_inc_ref_count ();
	g_free (message);
	
	switch (result) {
		case 0:
			return PED_EXCEPTION_UNHANDLED;
			break; 
		case DIALOG_FIX:
			return PED_EXCEPTION_FIX;
			break;
		case DIALOG_YES:
			return PED_EXCEPTION_YES;
			break;
		case DIALOG_NO:
			return PED_EXCEPTION_NO;
			break;
		case DIALOG_OK:
			return PED_EXCEPTION_OK; 
			break;
		case DIALOG_RETRY:
			return PED_EXCEPTION_RETRY;
			break;
		case DIALOG_IGNORE:
			return PED_EXCEPTION_IGNORE;
			break;
		case DIALOG_CANCEL:
			return PED_EXCEPTION_CANCEL;
			break;
		default:
			g_assert_not_reached ();
			break;
	}
	
	/* should have return before */
	g_assert_not_reached ();
	return PED_EXCEPTION_UNHANDLED;
}

static gpedInfo*
_gpedInfo_new                          (void)
{
	gpedInfo	*info;

	if ( !(info = g_malloc (sizeof (gpedInfo))) ) {
		g_printerr ("%s: allocating gpedInfo.\n",
		             g_strerror (errno));
		return NULL;
	}
	
	return info;
}

gpedInfo*
gped_init                              (gchar          *fstab_path,
                                        gint            devices_count,
                                        gchar          *devices[])
{
	gpedInfo	*info;
	
	printf ("gped init started\n");
	
	printf ("  ped init\n");
	ped_exception_set_handler (_exception_handler);
	if ( !ped_init () )
		return NULL;
	
	/*
	 * init gpedInfo
	 */
	printf ("  gpedinfo init\n");
	if ( !(info = _gpedInfo_new ()) )
		return NULL;
	info->ctreeParted = NULL;
	info->ctreeParted_scrolled = NULL;
	info->item_factory = NULL;
	info->dev_list = NULL;
	info->fstab_info = NULL;
	
	printf ("    build device list\n");
	if ( !(info->dev_list = _build_device_list (devices_count, devices)) ) {
		g_free (info);
		return NULL;
	}
	
	info->fstab_info = fstab_build_info (fstab_path,
	                                     info->dev_list);
	
	printf ("gped init ended\n");
	
	return info;
}

void
gped_done                              (void)
{

	
	ped_done ();

}


gboolean
gped_get_disk_changes                  (gpedInfo        *info,
                                        PedDisk         *disk)
{
	gpedDeviceInfo	*devInfo;
	
	g_return_val_if_fail (info, FALSE);
	g_return_val_if_fail (info->dev_list, FALSE);
	g_return_val_if_fail (disk, FALSE);
	
	if ( !(devInfo = _gpedDeviceInfo_get_by_disk (info->dev_list, disk)) )
		g_assert_not_reached ();
	
	return devInfo->modified;
}

gboolean
gped_get_changes                       (gpedInfo        *info)
{
	gpedDeviceInfo	*walk;

	g_return_val_if_fail (info, FALSE);
	g_return_val_if_fail (info->dev_list, FALSE);
	g_return_val_if_fail (info->fstab_info, FALSE);
	
	if (fstab_get_changes (info->fstab_info))
		return TRUE;
	
	for (walk = info->dev_list; walk; walk = walk->next) {
		if (walk->modified==TRUE)
			return TRUE;
	}
	return FALSE;
}

void
gped_change_done                       (gpedInfo        *info,
                                        PedDisk		*disk)
{
	gpedDeviceInfo	*devInfo;

	g_return_if_fail (info);
	g_return_if_fail (info->ctreeParted);
	g_return_if_fail (info->dev_list);
	g_return_if_fail (disk);
		
	if ( !(devInfo = _gpedDeviceInfo_get_by_disk (info->dev_list, disk)) )
		g_assert_not_reached ();
	
	devInfo->modified = TRUE;
	ctreeParted_refresh (GTK_CTREE (info->ctreeParted),
	                     info);
}

gboolean
gped_write_disk_changes               (gpedInfo         *info,
                                       PedDisk          *disk)
{
	gpedDeviceInfo	*devInfo;

	g_return_val_if_fail (info, FALSE);
	g_return_val_if_fail (disk, FALSE);

	if ( !ped_disk_write (disk) )
			return FALSE;
	
	if ( !(devInfo = _gpedDeviceInfo_get_by_disk (info->dev_list, disk)) )
		g_assert_not_reached ();
	
	devInfo->modified = FALSE;
	
	return TRUE;
}

gboolean
gped_write_changes                     (gpedInfo        *info)
{
	gpedDeviceInfo	*walk;
	
	g_return_val_if_fail (info, FALSE);
	g_return_val_if_fail (info->dev_list, FALSE);
	g_return_val_if_fail (info->fstab_info, FALSE);
	
	for (walk = info->dev_list; walk; walk = walk->next) {
		if ( !ped_disk_write (walk->disk) )
			return FALSE;
	}
	
	fstab_save (info->fstab_info);
	
	return TRUE;
}

gboolean
gped_insert                            (gpedInfo        *info,
                                        GtkContainer    *container)
{
	GtkWidget       *scrolledwindow;
	GtkWidget	*ctreeParted;
	GtkItemFactory  *item_factory;
	
	g_return_val_if_fail (info, FALSE);
	g_return_val_if_fail (container, FALSE);
	g_return_val_if_fail (GTK_IS_CONTAINER (container), FALSE);
	
	scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_add (GTK_CONTAINER (container), scrolledwindow);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	if ( !(ctreeParted = ctreeParted_insert (GTK_CONTAINER (scrolledwindow), info)) )
		return FALSE;

	info->ctreeParted = GTK_CTREE (ctreeParted);
	info->ctreeParted_scrolled = GTK_SCROLLED_WINDOW (scrolledwindow);

	if ( !(item_factory = popup_insert (info)) )
		return FALSE;
	
	info->item_factory = item_factory;
	
	return TRUE;
}
